//******************************************************************************
// Projeto - TESTE_DS2620 - testa termometro digital Dallas DS1620
// Desenvolvido por Mrcio Jos Soares
//
// Arquivo - teste_DS1620.c
//
// Realiza 1 leitura no DS1620 a cada 2 segundos (timer 0 ativo)
// e envia dado via canal serial RS232 - (TEMP->XX,XX)
// Comunicao: 9600, n, 8, 1
//
// Formato dos Dados:
// se RA0 = 1, envia resultado em Celsius (C)
// se RA1 = 0, envia resultado em Fahrenheit (F)
//
// LED1 pisca a 1 Hz (1 seg): indicao de ligado
//
// Microcontrolador utilizado: PIC16F628-04/P (no testado com verso "A")
// Perifrico - DS1620 Termomtro Digital Dallas
// Compilador: CCS C Compiler - verso PCM 3.15
//
// Pinos utilizados:
// RA0 - jumper de seleo para converso
// RA1 - LED
// RB0 - Reset do DS1620 (/RST)
// RB1 - RX
// RB2 - TX - sada dos dados
// RB3 - Clock do DS1620 (CLK) (protocolo Dallas)
// RB4 - Dados do DS1620 (DO) (protocolo Dallas)
//
// Obs:  utilizado MAX-232 para drive TTL/RS-232
//       e resistor de pull-up para pino RS
//
// Obs2: A comunicao entre o PIC e o DS16 20 foi feita "manualmente"
//       para uma melhor compreenso do seu funcionamento.
//******************************************************************************

//******************************************************************************
// Arquivos includos neste mdulo
//******************************************************************************
#include "teste_DS1620.h"

//******************************************************************************
// Funo para escrever dados no DS1620
// Entradas - valor a escrever (byte a escrever)
// Saidas   - nenhuma
//******************************************************************************
void escreve_DS1620(unsigned char dado){

   int i,meubit;

   restart_WDT();					//zera WDT

   for (i=0;i<8;i++){
      restart_WDT();				//zera WDT

      if (bit_test(dado, i))     //se bit = 1, insere 1
         output_bit(DADOS, 1);
      else
         output_bit(DADOS, 0);   //se bit = 0, insere 0

      output_low(CLK);        //abaixa pino de clock
      output_high(CLK);       //levanta clock
   }

   restart_WDT();					//zera WDT
}

//******************************************************************************
// Funo para ler dado do DS1620
// Entradas - nenhuma
// Saidas   - nenhuma
//******************************************************************************
void le_DS1620(void){

   int i;                     //varivel para contador
   dado_rec=0;                //zera dado

   restart_WDT();					//zera WDT

   //inicia as converses
   output_high(CLK);          //levanta clock
   output_high(RST);          //habilita DS1620
   escreve_DS1620(DS_INIT);   //inicia DS1620 (converses)
   output_low(RST);           //desabilita DS1620
   //delay_ms(200);

   output_high(CLK);          //levanta clock
   output_high(RST);          //habilita DS1620
   escreve_DS1620(DS_TEMP);   //envia comando para DS1620, solicitando leitura

   set_tris_A(0b00000101);		//configura pinos do port A como sada,
                              //exceto RA0 e RA2

   output_high(RST);          //habilita DS1620
   for(i=0;i<9;i++){          //so 9 bits a ler
      restart_WDT();				//zera WDT
      dado_rec=dado_rec/2;    //gira dado a direita
      output_low(CLK);        //abaixa clock

      if(input(DADOS))        //se bit igual a 1
         dado_rec=dado_rec | 0x0100;   //marca 1 no bit8
      else
         dado_rec=dado_rec & ~0x0100;  //seno desmarca

      output_high(CLK);       //levanta clock
   }
   output_low(RST);           //desabilita DS1620

   set_tris_A(0b00000001);		//configura pinos do port A como sada,
                              //exceto RA0

   output_high(CLK);          //levanta clock
   output_high(RST);          //habilita DS1620
   escreve_DS1620(DS_STOP);   //para converses no DS1620
   output_low(RST);           //desabilita DS1620

   restart_WDT();					//zera WDT
}

//******************************************************************************
// Funo para ler a configurao do DS1620
// Ajuda na depurao do programa e verificao do DS1620
//
// Entradas - nenhuma
// Saidas   - nenhuma (reflete dados diretamente na serial
//******************************************************************************
void le_configDS1620(void){

   unsigned char aux, i;

   restart_WDT();					//zera WDT

   delay_ms(100);
   escreve_DS1620(LECONF);    //prepara para ler configurao

   set_tris_A(0b00000101);		//configura pinos do port A como sada,
                              //exceto RA0 e RA2
   output_high(RST);          //habilita DS1620

   for(i=0;i<7;i++){          //so 8 bits
      restart_WDT();				//zera WDT
      aux=aux/2;              //gira dado a direita
      output_low(CLK);        //abaixa clock
      delay_us(5);

      if(input(DADOS))
         aux=aux | 0x80;      //ltimo bit = 1
      else
         aux=aux & ~0x80;     //ltimo bit=0

      output_high(CLK);       //levanta clock
   }

   set_tris_A(0b00000001);		//configura pinos do port A como sada,
                              //exceto RA0
   output_low(RST);           //desabilita DS1620

   //escreve o setup para DS1620
   //printf("Config->%u\n\r",aux);

   restart_WDT();					//zera WDT
}

//******************************************************************************
// Funo para configurar o DS1620
// Entradas - nenhuma
// Saidas   - nenhuma
//******************************************************************************
void config_DS1620(void){


   restart_WDT();					//zera WDT

   output_low(RST);           //deabilita DS1620

   //avisa que vai escrever dado no registrador de config.
   output_high(CLK);          //levanta clock
   output_high(RST);          //habilita DS1620
   escreve_DS1620(ESCRCONF);  //entra no modo configurao
   escreve_DS1620(DS_CPU+DS_CONT);   //modo CPU + modo continuo = 0x02
   output_low(RST);
   delay_ms(200);

   //inicia as converses
   output_high(CLK);          //levanta clock
   output_high(RST);          //habilita DS1620
   escreve_DS1620(DS_INIT);   //inicia DS1620 (converses)
   output_low(RST);           //desabilita DS1620
   delay_ms(200);

   output_high(CLK);          //levanta clock
   output_high(RST);          //habilita DS1620
   escreve_DS1620(DS_STOP);   //para converses no DS1620
   output_low(RST);           //desabilita DS1620
   delay_ms(200);
   //le_configDS1620();         //apenas para depurao

   conf1=true;                //setup pronto, leituras podem iniciar
   restart_WDT();					//zera WDT
}

//******************************************************************************
// Funo para converter valor recebido do CI DS1620
// recebe 	- tipo de converso (0 - Celsius, 1 - Fahrenheit)
// retorna 	- nada
//******************************************************************************
float converte (unsigned char tipo){

   float aux;

   restart_WDT();                //restaura WDT

   sinal='+';                         //converso sempre positiva
   if ((dado_rec & 0x100) != 0x00){   //se o bit9 = 1 ento valor passado, negativo
      sinal='-';                      //converso  negativa
      dado_rec = (~dado_rec+1) & 0xFF;//inverte dado e soma 1: complemento de dois
   }                                  //

   if (tipo)                           //se converso  Celsius
      aux=dado_rec * 0.5;              //multiplica valor passado por 0,5 (preciso)
   else                                //se fahrenheit
      aux=((dado_rec * 0.5)* 1.8)+32;  //multiplica por 0,5, depois 1,8 e soma 32

   restart_WDT();                      //restaura WDT

   return(aux);                        //retorna valor
}

//******************************************************************************
// Funo para tratar a interrupo para dado recebido RS-232
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_RDA
RDA_isr() {

   static char carac;

   restart_WDT();                //restaura WDT

   carac = rxreg;                //colhe dado do canal serial
   printf("%c",carac);           //envia eco de caracter recebido

   restart_WDT();                //restaura WDT
}

//******************************************************************************
// Funo para tratar a interrupo do timer0
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_timer0
void trata_int_t0(){

   static int16 contador;
   float envia;


   restart_WDT();							//zera WDT

   //reinicia o timer0 em 131, mas a contagem que j passou
   set_timer0(131+get_timer0());
   contador++;
   //verifica se j ocorreram 125 interrupes
   //125 Hz / 250 = 0,5 Hz = 1/0,5 = 2 segundos
   if (contador == 250){
      contador = 0;

      if (conf1){                      //se DS1620 j foi configurado
         le_DS1620();                  //le dado do DS1620
         envia=converte(convert);      //converte leitura
         printf("TEMP->%c%2.2f", sinal, envia); //envia dados
         if (convert)
            printf("C");
         else
            printf("F");
         printf("\n\r");
     }
   }

   restart_WDT();							//zera WDT
}

//******************************************************************************
// Funo para tratar a interrupo do timer1
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_timer1
void trata_int_t1(){

	static int conta;
	static boolean led1;

	restart_WDT();					//zera WDT

	//reinicia timer1 com 3096, mais contagem que passou
	set_timer1(3096+get_timer1());
	conta++;
	if (conta == 2){
		conta=0;
		led1=!led1;
		output_bit(LED,led1);
		}

	restart_WDT();					//zera WDT
}

//******************************************************************************
// Funo principal
//******************************************************************************
void main(void) {

   set_tris_A(0b00000001);		//configura pinos do port A como sada,
                              //exceto RA0
   set_tris_B(0b00000010);		//configura pinos do port B como sada,
                              //exceto RB1

   setup_comparator(nc_nc_nc_nc); //desabilita comparadores internos
	setup_wdt(WDT_2304MS);			 //habilita watch dog timer a cada 2 segundos
	restart_WDT();						//zera WDT

   //timer 0 habilitado
   //[(4MHz/4)/64]/125 = 125 Hz -> valor da frequencia escolhida
   //para obter o valor de inicializao do timer deve-se fazer (256-125)=131
   setup_timer_0(RTCC_INTERNAL | RTCC_DIV_64);
   set_timer0(131);

	//timer 1 habilitado
	// [(4MHz/4)/8]/65636 = 1.90Hz -> menor frequencia possvel
	//valor inicial do timer1 (65536-62500)=3036
   setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);
	set_timer1(3036);
	restart_WDT();								//zera WDT

	//timer2 desabilitado
	setup_timer_2(T2_DISABLED,0,1);
	restart_WDT();								//zera WDT

   //habilita interrupes
   enable_interrupts(INT_RDA);         //habilita interrupo RS-232 (rec)
   enable_interrupts(INT_TIMER0);      //habilita interrupo do timer0
   enable_interrupts(INT_TIMER1);      //habilita interrupo do timer1
   enable_interrupts(GLOBAL);          //habilita interrupo global
   restart_WDT();							   //zera WDT

   output_low(RST);                    //mantem CI DS1620 desabilitado

   printf("P\n\r");                    //envia sinal de prontido via RS-232
   config_DS1620();                    //configura DS1620

   restart_WDT();	                     //zera WDT

//******************************************************************************
// Inicio do lao principal
// A leitura do DS1620 e o envio do dado  feita via interrupo do timer 0
// O piscar do LED  controlado via interrupo do timer 1
// A recepo e retorno do caracter de eco  feita via int RDA (rec RS-232)
//******************************************************************************

   while(true){                        //faz eternamente

      if (input(JUMPER))               //apenas testa o jumper
         convert=1;                    //envia resultado em Celsius
      else
         convert=0;                    //envia resultado em Fahrenheit

      delay_ms(100);                  //l jumper a cada 100 mili segundos
      restart_WDT();

   }
}
